@@ -100,7 +100,7 @@ bootstrap_modsem.modsem_da <- function(model,
                                        FUN = "coef",
                                        R = 1000L,
                                        P.max = 1e5,
-                                       type = c("nonparametric", "parameteric"),
+                                       type = c("nonparametric", "parametric"),
                                        verbose = interactive(),
                                        calc.se = FALSE,
                                        optimize = FALSE,
@@ -108,18 +108,23 @@ bootstrap_modsem.modsem_da <- function(model,
   type <- tolower(type)
   type <- match.arg(type)
 
-  data     <- as.data.frame(modsem_inspect(model, what = "data"))
+  data.mat <- modsem_inspect(model, what = "data")
+  data     <- as.data.frame(data.mat)
   ovs      <- colnames(data)
-  N        <- nrow(data)
-  P.ceil   <- P.max < N * R
+  N        <- NROW(data)
   P        <- min(P.max, N * R)
   parTable <- parameter_estimates(model)
 
   warnif(P.max <= N, "`P.max` is less than `N`!")
 
+  cluster <- attr(data.mat, "cluster")
+
+  stopif(!is.null(cluster) && type != "nonparametric",
+         "cluster bootstrap is only available with `type=\"nonparametric\"`!")
+
   population <- switch(type,
-    parameteric    = simulateDataParTable(parTable, N = P, colsOVs = ovs)$oV,
-    nonparametric = data[sample(nrow(data), P, replace = TRUE), ],
+    parametric   = simulateDataParTable(parTable, N = P, colsOVs = ovs)$oV,
+    nonparametric = data,
     stop2("Unrecognized type!\n")
   )
 
@@ -127,6 +132,7 @@ bootstrap_modsem.modsem_da <- function(model,
   argList$calc.se      <- calc.se
   argList$verbose      <- verbose
   argList$model.syntax <- model$model$syntax
+  argList$cov.syntax   <- model$model$covModel$syntax
   argList$method       <- model$method
 
   if (!optimize) {
@@ -137,7 +143,7 @@ bootstrap_modsem.modsem_da <- function(model,
     argList$optimize     <- TRUE
   }
 
-  f0 <- FUN(model)
+  f0 <- do.call(FUN, args = list(model))
   FUN.VEC <- is.vector(f0) || inherits(f0, "ModsemVector")
 
   if (FUN.VEC) {
@@ -152,7 +158,7 @@ bootstrap_modsem.modsem_da <- function(model,
     printedLines <- utils::capture.output(split = TRUE, {
       if (verbose) printf("Bootstrap %d/%d...\n", i, R)
 
-      sample_i  <- population[sample(P, N), ]
+      sample_i  <- resample(population, n.out = N, cluster = cluster)
       argList_i <- c(argList, list(data = sample_i))
 
       fit_i <- tryCatch(
@@ -185,6 +191,7 @@ bootstrap_modsem.modsem_da <- function(model,
 #' @param verbose Should progress information be printed to the console?
 #' @param data Dataset to be resampled.
 #' @param FUN.args Arguments passed to \code{FUN}
+#' @param cluster.boot Variable to cluster bootstrapping by
 #'
 #' @details This is a more general version of \code{boostrap_modsem} for
 #'   bootstrapping \code{modsem} functions, not modsem objects.
@@ -230,6 +237,7 @@ bootstrap_modsem.function <- function(model = modsem,
                                       R = 1000L,
                                       verbose = interactive(),
                                       FUN.args = list(),
+                                      cluster.boot = NULL,
                                       ...) {
   MODELFUN <- model # rename for convenience
 
@@ -240,11 +248,18 @@ bootstrap_modsem.function <- function(model = modsem,
 
   ERROR <- \(e) {warning2(e, immediate. = FALSE); NULL}
 
+  if (!is.null(cluster.boot)) {
+    stopif(length(cluster.boot) > 1L, "`cluster.boot` must be of length 1!")
+    stopif(!cluster.boot %in% colnames(data), "`cluster.boot` must be a column in `data`!")
+    cluster.values <- data[[cluster.boot]]
+
+  } else cluster.values <- NULL
+
   for (i in seq_len(R)) {
     printedLines <- utils::capture.output(split = TRUE, {
       if (verbose) printf("Bootstrap %d/%d...\n", i, R)
 
-      sample_i  <- data[sample(N, N, replace = TRUE), ]
+      sample_i  <- resample(data, n.out = NROW(data), cluster = cluster.values)
       argList_i <- c(list(data = sample_i), args)
 
       fit_i <- tryCatch(
@@ -278,3 +293,24 @@ bootstrap_modsem.function <- function(model = modsem,
 
   out
 }
+
+
+resample <- function(df, n.out = NROW(df), cluster = NULL, replace = TRUE) {
+  df.orig   <- as.data.frame(df)
+
+  if (is.null(cluster)) {
+    idx <- sample(NROW(df.orig), size = n.out, replace = replace)
+    return(df.orig[idx, , drop = FALSE])
+  }
+
+  stopif(length(cluster) != NROW(df.orig), "Cluster must be of same lenght as data!")
+
+  clusters <- unique(cluster)
+  G <- length(clusters)
+
+  clusters.sample <- sample(clusters, size = G, replace = replace)
+
+  purrr::list_rbind(
+    lapply(clusters.sample, FUN = \(ci) df.orig[cluster==ci, , drop=FALSE])
+  )
+}
