@@ -29,13 +29,13 @@
 #' # Double centering approach
 #' est_h1 <- modsem(m1, oneInt, method = "dblcent")
 #' est_h0 <- estimate_h0(est_h1, oneInt)
-#' 
+#'
 #' compare_fit(est_h1 = est_h1, est_h0 = est_h0)
-#' 
+#'
 #' # Constrained approach
 #' est_h1 <- modsem(m1, oneInt, method = "ca")
 #' est_h0 <- estimate_h0(est_h1, oneInt)
-#' 
+#'
 #' compare_fit(est_h1 = est_h1, est_h0 = est_h0)
 #' }
 #' @export
@@ -49,25 +49,25 @@ estimate_h0 <- function(object, warn_no_interaction = TRUE, ...) {
 estimate_h0.modsem_da <- function(object, warn_no_interaction = TRUE, ...) {
   argList    <- object$args
   parTable   <- object$originalParTable
-  data       <- object$data
+  data       <- object$data$data.full
   method     <- object$method
   cov.syntax <- object$model$covModel$syntax
 
-  newArgList <- list(...) 
+  newArgList <- list(...)
   newArgNames <- intersect(names(argList), names(newArgList))
-  argList[newArgNames] <- newArgList[newArgNames] 
+  argList[newArgNames] <- newArgList[newArgNames]
 
   tryCatch({
     # labels should probably be replaced instead...
     strippedParTable <- removeUnknownLabels(parTable[!grepl(":", parTable$rhs), ])
-    
+
     if (NROW(strippedParTable) == NROW(parTable)) {
       warnif(warn_no_interaction, "No interaction terms found in the model. ",
              "The baseline model is identical to the original model, ",
              "and won't be estimated!")
       return(NULL)
-    } 
-    
+    }
+
     syntax <- parTableToSyntax(strippedParTable)
     argList <- c(
       list(model.syntax = syntax, data = data, method = method,
@@ -85,13 +85,13 @@ estimate_h0.modsem_da <- function(object, warn_no_interaction = TRUE, ...) {
 
 
 #' @describeIn estimate_h0 Estimate baseline model for \code{\link{modsem_pi}} objects
-#' @param reduced Should the baseline model be a reduced version of the model? 
+#' @param reduced Should the baseline model be a reduced version of the model?
 #'   If \code{TRUE}, the latent product term and its (product) indicators are kept in the model,
 #'   but the interaction coefficients are constrained to zero. If \code{FALSE}, the
-#'   interaction terms are removed completely from the model. Note that the models will no longer be 
+#'   interaction terms are removed completely from the model. Note that the models will no longer be
 #'   nested, if the interaction terms are removed from the model completely.
 #' @export
-estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE, 
+estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
                                   reduced = TRUE, ...) {
   pars       <- parameter_estimates(object)
   input      <- object$input
@@ -99,30 +99,30 @@ estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
   syntax     <- input$syntax
   data       <- input$data
   parTable   <- input$parTable
- 
+
   # Get arguments
   lavArgs    <- input$lavArgs
   modsemArgs <- input$modsemArgs
   lavArgs$meanstructure <- any(pars$op == "~1")
   argList    <- c(modsemArgs, lavArgs)
 
-  newArgList <- list(...) 
+  newArgList <- list(...)
   newArgNames <- intersect(names(argList), names(newArgList))
-  argList[newArgNames] <- newArgList[newArgNames] 
+  argList[newArgNames] <- newArgList[newArgNames]
 
   if (!is.null(lavArgs$cluster)) {
     warning2("Baseline model cannot (yet) be estimated with clustering!", .immediate. = FALSE)
     return(NULL)
-  } 
+  }
 
   tryCatch({
     if (!parTableHasInteraction(parTable)) {
-      warnif(warn_no_interaction, 
+      warnif(warn_no_interaction,
              "No interaction terms found in the model. ",
              "The baseline model is identical to the original model, ",
              "and won't be estimated!")
       return(NULL)
-   
+
     } else if (parTableHasHigherOrderInteraction(parTable)) {
       warning2("Unable to estimate baseline model for models with higher-order interaction terms!",
                immediate. = FALSE)
@@ -132,7 +132,7 @@ estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
     isInteractionTerm <- grepl(":", parTable$rhs)
     constrainedParTable <- parTable
 
-    # If there are any label modifiers, we must not remove them completely 
+    # If there are any label modifiers, we must not remove them completely
     # from the model, as they might be used elsewhere in the model
     preExistingMod <- constrainedParTable[isInteractionTerm, "mod"]
     preExistingMod <- preExistingMod[preExistingMod != ""]
@@ -140,11 +140,11 @@ estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
 
     # Redine the labels as zero, if they exist
     if (length(preExistingLab)) {
-      redefinedLabels <- data.frame(lhs = preExistingLab, op  = ":=", rhs = "0", 
+      redefinedLabels <- data.frame(lhs = preExistingLab, op  = ":=", rhs = "0",
                                     mod = preExistingLab)
       constrainedParTable <- rbind(redefinedLabels, constrainedParTable)
     }
-    
+
     # We don't want to remove the interaction term, since we want the baseline model
     # to have the same variables as the original model (inluding the product indicators).
     # However, we want to constrain the interaction terms to zero.
@@ -152,7 +152,7 @@ estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
     isInteractionTerm <- grepl(":", constrainedParTable$rhs) # recompute as parTable might have changed
     if (reduced) constrainedParTable[isInteractionTerm, "mod"] <- "0"
     else         constrainedParTable <- constrainedParTable[!isInteractionTerm, ]
-    
+
     syntax <- parTableToSyntax(constrainedParTable)
     argList <- c(list(model = syntax, data = data, method = method), argList)
 
@@ -166,7 +166,7 @@ estimate_h0.modsem_pi <- function(object, warn_no_interaction = TRUE,
     fit
   },
   error = function(e) {
-    warning2("Baseline model could not be estimated: ", e$message, 
+    warning2("Baseline model could not be estimated: ", e$message,
              immediate. = FALSE)
     NULL
   })
