@@ -35,15 +35,20 @@ specifyModelDA <- function(syntax = NULL,
     cov.syntax <- parTableToSyntax(parTableCovModel)
   }
 
-  checkParTableDA(parTable)
+  checkParTableDA(parTable, method = method)
   # additions to lavaan-syntax for optimizer
   lavOptimizerSyntaxAdditions <- ""
 
+  # General Information
+  higherOrderLVs <- getHigherOrderLVs(parTable)
+  indsHigherOrderLVs <- getIndsLVs(parTable, lVs = higherOrderLVs, isOV = FALSE)
+  ovs <- getOVs(parTable)
+
   # endogenous variables (etas)model
   etas    <- getSortedEtas(parTable, isLV = TRUE, checkAny = TRUE)
   numEtas <- length(etas)
 
-  indsEtas    <- getIndsLVs(parTable, etas)
+  indsEtas    <- getIndsLVs(parTable, lVs = etas, isOV = TRUE, ovs = ovs)
   numIndsEtas <- vapply(indsEtas, FUN.VALUE = vector("integer", 1L),
                         FUN = length)
   allIndsEtas    <- unique(unlist(indsEtas))
@@ -61,7 +66,7 @@ specifyModelDA <- function(syntax = NULL,
   xis <- omegaAndSortedXis$sortedXis # get sorted xis according to interaction terms
   nonLinearXis <- omegaAndSortedXis$nonLinearXis
 
-  indsXis    <- getIndsLVs(parTable, xis)
+  indsXis    <- getIndsLVs(parTable, lVs = xis, isOV = TRUE, ovs = ovs)
   numIndsXis <- vapply(indsXis, FUN.VALUE = vector("integer", 1L),
                        FUN = length)
   allIndsXis    <- unique(unlist(indsXis))
@@ -84,8 +89,18 @@ specifyModelDA <- function(syntax = NULL,
   lavOptimizerSyntaxAdditions <- paste0(lavOptimizerSyntaxAdditions,
                                         listTauX$syntaxAdditions)
 
-  listThetaDelta <- constructTheta(xis, indsXis, parTable = parTable,
-                                   auto.fix.single = auto.fix.single)
+  checkResCovX_Y(parTable = parTable, allIndsXis = allIndsXis,
+                 allIndsEtas = allIndsEtas, method = method)
+
+  if (method == "qml") {
+    listThetaDelta <- constructTheta(xis, indsXis, parTable = parTable,
+                                     auto.fix.single = auto.fix.single)
+  } else {
+    listThetaDelta <- constructTheta(c(xis, etas), c(indsXis, indsEtas),
+                                     parTable = parTable,
+                                     auto.fix.single = auto.fix.single)
+  }
+
   thetaDelta      <- listThetaDelta$numeric
   thetaLabelDelta <- listThetaDelta$label
 
@@ -102,18 +117,26 @@ specifyModelDA <- function(syntax = NULL,
   lavOptimizerSyntaxAdditions <- paste0(lavOptimizerSyntaxAdditions,
                                         listTauY$syntaxAdditions)
 
-  listThetaEpsilon <- constructTheta(etas, indsEtas, parTable = parTable,
-                                     auto.fix.single = auto.fix.single)
+  if (method == "qml") {
+    listThetaEpsilon <- constructTheta(etas, indsEtas, parTable = parTable,
+                                       auto.fix.single = auto.fix.single)
+  } else {
+    listThetaEpsilon <- constructTheta(NULL, NULL, parTable = parTable,
+                                       auto.fix.single = auto.fix.single)
+  }
+
   thetaEpsilon      <- listThetaEpsilon$numeric
   thetaLabelEpsilon <- listThetaEpsilon$label
 
   # structural model
   Ieta         <- diag(numEtas) # used for (B^-1 = (Ieta - gammaEta)^-1)
-  listGammaXi  <- constructGamma(etas, xis, parTable = parTable)
+  listGammaXi  <- constructGamma(etas, xis, parTable = parTable,
+                                 auto.fix.first = auto.fix.first)
   gammaXi      <- listGammaXi$numeric
   labelGammaXi <- listGammaXi$label
 
-  listGammaEta  <- constructGamma(etas, etas, parTable = parTable)
+  listGammaEta  <- constructGamma(etas, etas, parTable = parTable,
+                                  auto.fix.first = auto.fix.first)
   gammaEta      <- listGammaEta$numeric
   labelGammaEta <- listGammaEta$label
 
@@ -269,7 +292,11 @@ specifyModelDA <- function(syntax = NULL,
       kOmegaEta     = getK_NA(omegaEtaXi, labelOmegaEtaXi),
       nonLinearXis  = nonLinearXis,
       mean.observed = mean.observed,
-      has.interaction = NROW(intTerms) > 0L,
+
+      has.interaction    = NROW(intTerms) > 0L,
+      higherOrderLVs     = higherOrderLVs,
+      indsHigherOrderLVs = indsHigherOrderLVs,
+
       lavOptimizerSyntaxAdditions = lavOptimizerSyntaxAdditions
     ),
 
@@ -287,7 +314,7 @@ specifyModelDA <- function(syntax = NULL,
 
   model$constrExprs <- getConstrExprs(parTable, model$covModel$parTable)
   if (createTheta) {
-    listTheta         <- createTheta(model)
+    listTheta         <- createTheta(model, parTable.in = parTable)
     model             <- c(model, listTheta)
     model$freeParams  <- length(listTheta$theta)
     model$info$bounds <- getParamBounds(model)
