@@ -3,8 +3,8 @@
 #' @param model.syntax lavaan/modsem syntax
 #' @param data dataset
 #' @param estimator estimator argument passed to \code{Mplus}.
-#' @param type type argument passed to \code{Mplus}.
 #' @param cluster cluster argument passed to \code{Mplus}.
+#' @param type type argument passed to \code{Mplus}.
 #' @param algorithm algorithm argument passed to \code{Mplus}.
 #' @param processors processors argument passed to \code{Mplus}.
 #' @param integration integration argument passed to \code{Mplus}.
@@ -53,8 +53,8 @@
 modsem_mplus <- function(model.syntax,
                          data,
                          estimator = "ml",
-                         type = "random",
                          cluster = NULL,
+                         type = ifelse(is.null(cluster), yes = "random", no = "complex"),
                          algorithm = "integration",
                          processors = 2,
                          integration = 15,
@@ -80,8 +80,17 @@ modsem_mplus <- function(model.syntax,
 
   # Abbreviate variable names
   names <- unique(c(parTable$rhs, parTable$lhs))
-  names <- names[!grepl(":", names)]
-  abbreviated <- abbreviate(names, minlength = 8L, strict = TRUE)
+  names.xz   <- names[grepl(":", names)]
+  names.nlin <- unique(unlist(stringr::str_split(names.xz, pattern = ":")))
+
+  names     <- names[!grepl(":", names)]
+  names.lin <- setdiff(names, names.nlin)
+
+  # we need some part of both x and z to be available in the intTerm name
+  # so variables in int terms must be less than width = 8
+  abbreviated.lin  <- abbreviate(names.lin, minlength = 8L, strict = TRUE)
+  abbreviated.nlin <- abbreviate(names.nlin, minlength = 6L, strict = TRUE)
+  abbreviated <- c(abbreviated.lin, abbreviated.nlin)
 
   # Abbreviate names in intTerms
   intTerms <- unique(parTable$rhs[grepl(":", parTable$rhs)])
@@ -93,12 +102,13 @@ modsem_mplus <- function(model.syntax,
   abbreviated <- c(abbreviated, newIntTerms)
 
   # Fix names in parTable
-  rmask <- parTable$rhs != ""
+  rmask <- parTable$rhs != "" & !parTable$op %in% c(":=", "==", "<", ">")
   lmask <- parTable$lhs != ""
   parTable$rhs[rmask] <- abbreviated[parTable$rhs[rmask]]
   parTable$lhs[lmask] <- abbreviated[parTable$lhs[lmask]]
 
   # Fix names in data
+  data  <- as.data.frame(data)
   dmask <- colnames(data) %in% names(abbreviated)
   colnames(data)[dmask] <- abbreviated[colnames(data)[dmask]]
 
@@ -122,11 +132,13 @@ modsem_mplus <- function(model.syntax,
 
   } else VARIABLE <- NULL
 
+  usevariables <- intersect(c(cluster, indicators), colnames(data))
+
   # Estimate model
   model <- MplusAutomation::mplusObject(
     TITLE = "Running Model via Mplus",
     VARIABLE = VARIABLE,
-    usevariables = c(cluster, indicators),
+    usevariables = usevariables,
     ANALYSIS = paste0(
       paste(paste("estimator =", estimator),
             paste("type =", type),
@@ -134,9 +146,10 @@ modsem_mplus <- function(model.syntax,
             paste("processors =", processors),
             paste("integration = ", integration),
             sep = ";\n"), ";\n"), # add final ";"
-    MODEL = parTableToMplusModel(parTable, ...),
+    MODEL = parTableToMplusModel(parTable, ignoreConstraints = TRUE, ...),
+    MODELCONSTRAINT = parTableToMplusModelConstraints(parTable, ...),
     OUTPUT = OUTPUT,
-    rdata = data[c(cluster, indicators)],
+    rdata = data[usevariables],
   )
 
   results <- MplusAutomation::mplusModeler(model,
@@ -146,7 +159,8 @@ modsem_mplus <- function(model.syntax,
   mplusParTable <- mplusTableToParTable(coefsTable,
                                         intTerms = intTerms,
                                         intTermsMplus = intTermsMplus,
-                                        indicators = indicators)
+                                        indicators = indicators,
+                                        parTable.in = parTable)
 
   # coef and vcov
   TECH1 <- MplusAutomation::get_results(results, element = "tech1")
@@ -222,8 +236,13 @@ xwith <- function(elems) {
 }
 
 
-parTableToMplusModel <- function(parTable) {
-  # INTERACTIONEXPRESSIOns
+parTableToMplusModel <- function(parTable, ignoreConstraints = FALSE) {
+  if (ignoreConstraints) {
+    parTable <- parTable[!parTable$op %in% c(":=", "==", "<", ">"),
+                         , drop = FALSE]
+  }
+
+  # Interaction expressions
   interactions <- parTable[grepl(":", parTable$rhs), "rhs"]
   elemsInInts <- stringr::str_split(interactions, ":")
   newRows <- lapply(elemsInInts, FUN = xwith) |>
@@ -255,6 +274,29 @@ parTableToMplusModel <- function(parTable) {
   stringr::str_remove_all(out, ":")
 }
 
+parTableToMplusModelConstraints <- function(parTable) {
+  constraints <- parTable[parTable$op %in% c(":=", "==", "<", ">"),
+                          , drop = FALSE]
+
+  if (!NROW(constraints))
+    return(NULL)
+
+  out <- ""
+  for (i in seq_len(NROW(constraints))) {
+    lhs <- constraints$lhs[i]
+    op  <- constraints$op[i]
+    rhs <- constraints$rhs[i]
+
+    if (op == ":=")
+      out <- paste0(out, "NEW(", lhs, ");\n")
+
+    mop <- ifelse(op %in% c(":=", "=="), yes = "=", no = op)
+    out <- paste0(out, sprintf("%s %s %s;\n", lhs, mop, rhs))
+  }
+
+  out
+}
+
 
 replaceLavOpWithMplus <- function(op) {
   vapply(op,
@@ -276,7 +318,8 @@ switchLavOpToMplus <- function(op) {
 mplusTableToParTable <- function(coefsTable,
                                  indicators,
                                  intTerms,
-                                 intTermsMplus) {
+                                 intTermsMplus,
+                                 parTable.in = NULL) {
   coefsTable <- rename(coefsTable, Label = "label",
                        se = "std.error", pval = "p.value")
   coefsTable$label <- stringr::str_remove_all(coefsTable$label, pattern = " ")
@@ -297,7 +340,7 @@ mplusTableToParTable <- function(coefsTable,
     cbind(coefsTable[measCoefNames, ])
 
   # Structural Model
-  measrRemoved <- coefsTable[!measCoefNames, ]
+  measrRemoved <- coefsTable[!measCoefNames, , drop = FALSE]
   patternStruct <- "<-(?!>|Intercept)"
   structCoefNames <- grepl(patternStruct, measrRemoved$label, perl = TRUE)
 
@@ -318,7 +361,7 @@ mplusTableToParTable <- function(coefsTable,
   }
 
   # Variances and Covariances
-  structMeasrRemoved <- measrRemoved[!structCoefNames, ]
+  structMeasrRemoved <- measrRemoved[!structCoefNames, , drop = FALSE]
   patternCovVar <- "<->"
   covVarCoefNames <- grepl(patternCovVar, structMeasrRemoved$label, perl = TRUE)
   covVarLhs <- stringr::str_split_i(structMeasrRemoved$label[covVarCoefNames],
@@ -329,7 +372,7 @@ mplusTableToParTable <- function(coefsTable,
     cbind(structMeasrRemoved[covVarCoefNames, ])
 
   # Intercepts
-  covStructMeasrRemoved <- structMeasrRemoved[!covVarCoefNames, ]
+  covStructMeasrRemoved <- structMeasrRemoved[!covVarCoefNames, , drop = FALSE]
   patternIntercept <- "<-Intercept"
   interceptNames <- grepl(patternIntercept, covStructMeasrRemoved$label, perl = TRUE)
   interceptLhs <- stringr::str_split_i(covStructMeasrRemoved$label[interceptNames],
@@ -337,11 +380,50 @@ mplusTableToParTable <- function(coefsTable,
   interceptModel <- data.frame(lhs = interceptLhs, op = "~1", rhs = "") |>
     cbind(covStructMeasrRemoved[interceptNames, ])
 
-  mplusParTable <- rbind(measModel, structModel, covVarModel, interceptModel)
+  # Custom / Remaining
+  intCovStructMeasrRemoved <- covStructMeasrRemoved[!interceptNames, , drop = FALSE]
+
+  if (NROW(intCovStructMeasrRemoved)) {
+    customModel <- data.frame(lhs = intCovStructMeasrRemoved$label, op = ":=", rhs = "") |>
+      cbind(intCovStructMeasrRemoved)
+
+    if (!is.null(parTable.in)) {
+      lrCustom <- parTable.in[parTable.in$op == ":=", c("lhs", "rhs"), drop = FALSE]
+      lrCustom$lhs <- toupper(lrCustom$lhs)
+
+      notRhs   <- colnames(customModel) != "rhs"
+      customModel <- merge(x = customModel[notRhs],
+                           y = lrCustom, by = "lhs",
+                           all.x = TRUE, all.y = FALSE)
+    }
+
+  } else customModel <- NULL
+
+  # Combine
+  mplusParTable <- rbind(measModel, structModel, covVarModel, interceptModel,
+                         customModel)
   mplusParTable [c("lhs", "rhs")] <-
     lapplyDf(mplusParTable[c("lhs", "rhs")], function(x)
              stringr::str_remove_all(x, " "))
 
+  if (!is.null(parTable.in) && any(parTable.in$mod != "")) {
+    LABELS <- parTable.in[parTable.in$mod != "", , drop = FALSE]
+    LABELS <- rename(LABELS[c("lhs", "op", "rhs", "mod")], mod = "label.lav")
+
+    mplusParTable$order <- seq_len(NROW(mplusParTable))
+    mplusParTable <- merge(x = mplusParTable,
+                           y = LABELS,
+                           by = c("lhs", "op", "rhs"),
+                           all.x = TRUE, all.y = FALSE)
+
+    match <- !is.na(mplusParTable$label.lav)
+    mplusParTable[match, "label"] <- mplusParTable[match, "label.lav"]
+
+    # clean up
+    mplusParTable <- mplusParTable[colnames(mplusParTable) != "label.lav"]
+    mplusParTable <- mplusParTable[order(mplusParTable$order), , drop = FALSE]
+  }
+
   mplusParTable$ci.lower <- mplusParTable$est - CI_WIDTH * mplusParTable$std.error
   mplusParTable$ci.upper <- mplusParTable$est + CI_WIDTH * mplusParTable$std.error
   mplusParTable$p.value[mplusParTable$p.value == 999] <- NA
@@ -354,12 +436,20 @@ mplusTableToParTable <- function(coefsTable,
 
 getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMplus) {
   spec   <- TECH1$parameterSpecification
-  nu     <- spec$nu
-  alpha  <- spec$alpha
-  lambda <- spec$lambda
-  beta   <- spec$beta
-  psi    <- spec$psi
-  theta  <- spec$theta
+
+  CPAR <- "THE.ADDITIONAL.PARAMETERS"
+  if (CPAR %in% names(spec)) custom <- spec[[CPAR]]$new_additional
+  else                       custom <- NULL
+
+  if ("X" %in% names(spec)) specX <- spec$X
+  else                      specX <- spec
+
+  nu     <- specX$nu
+  alpha  <- specX$alpha
+  lambda <- specX$lambda
+  beta   <- specX$beta
+  psi    <- specX$psi
+  theta  <- specX$theta
 
   setLabel <- function(out, label, id) {
     if (!length(label)) {
@@ -372,7 +462,7 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     out
   }
 
-  getReg <- function(M, row.lhs = TRUE, op = "~") {
+  getReg <- function(M, row.lhs = TRUE, op = "~", try.op = NULL) {
     out <- c()
 
     rows <- rownames(M)
@@ -380,7 +470,7 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     for (i in seq_len(NROW(M))) for (j in seq_len(NCOL(M))) {
       id <- as.integer(M[i, j])
 
-      if (id <= 0L)
+      if (is.na(id) || id <= 0L)
         next
 
       if (row.lhs) {
@@ -395,6 +485,14 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
                         parTable$lhs == lhs &
                         parTable$rhs == rhs, "label"]
 
+      if (!length(label) && !is.null(try.op)) {
+        # can either be =~, or ~, either way we must also
+        # switch from order of lhs and rhs
+        label <- parTable[parTable$op == try.op &
+                          parTable$lhs == rhs &
+                          parTable$rhs == lhs, "label"]
+      }
+
       out <- setLabel(out = out, label = label, id = id)
     }
 
@@ -409,7 +507,7 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     for (i in seq_along(T)) {
       id <- as.integer(T[i])
 
-      if (id <= 0L)
+      if (is.na(id) || id <= 0L)
         next
 
       lhs <- vars[i]
@@ -433,7 +531,7 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     for (i in seq_len(NROW(M))) for (j in seq_len(i)) {
       id <- as.integer(M[i, j])
 
-      if (id <= 0L)
+      if (is.na(id) || id <= 0L)
         next
 
       lhs <- cols[i]
@@ -449,6 +547,32 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     out
   }
 
+  getCustom <- function(M, op = ":=") {
+    if (is.null(M) || NROW(M) == 0L) return(NULL)
+
+    # assumes M has nrows(M)
+    warnif(NROW(M) > 1L,
+           "Expected parameter matrix for additional pars\n",
+           "to have a single row!", immediate. = FALSE)
+
+    cols <- colnames(M)
+    out  <- c()
+
+    for (i in seq_len(NCOL(M))) {
+      id <- as.integer(M[1L, i])
+
+      if (is.na(id) || id <= 0L)
+        next
+
+      lhs <- cols[i]
+
+      label <- parTable[parTable$op == op & parTable$lhs == lhs, "lhs"]
+      out <- setLabel(out = out, label = label, id = id)
+    }
+
+    out
+  }
+
   for (i in seq_along(intTerms)) {
     xzMplus  <- substr(intTermsMplus[[i]], start = 1L, stop = 8L)
     xzModsem <- intTerms[[i]]
@@ -461,13 +585,13 @@ getOrderedParameterLabelsMplus <- function(parTable, TECH1, intTerms, intTermsMp
     getIntercept(nu),
     getIntercept(alpha),
     getReg(lambda, row.lhs = FALSE, op = "=~"),
-    getReg(beta, row.lhs = TRUE, op = "~"),
+    getReg(beta, row.lhs = TRUE, op = "~", try.op = "=~"), # include "=~" for higher order models
     getCovariance(psi),
-    getCovariance(theta)
+    getCovariance(theta),
+    getCustom(custom)
   )
 
   out <- out[!duplicated(out)] # unique() removes labels
-
   sort(out)
 }
 
